@@ -1,19 +1,19 @@
 """
 transparent serialization of numpy/pandas data via jsonpickle.
-compatible to python2.7 and python3.3 and allows to serialize 
+compatible to python2.7 and python3.3 and allows to serialize
 between the two interpreters.
 
-majorly based on code and ideas of David Moss in his MIT licensed pdutils 
+majorly based on code and ideas of David Moss in his MIT licensed pdutils
 repository: https://github.com/drkjam/pdutils
 
 Note that the serialization/deserialization is not space-efficient
-due to the nature of json/jsonpickle. You could certainly save space 
+due to the nature of json/jsonpickle. You could certainly save space
 by compressing/decompressing the resulting json output if you need to.
 
 (C) David Moss, Holger Krekel 2014
 """
 
-__version__ = '0.2'
+__version__ = '0.3'
 import numpy as np
 import pandas as pd
 
@@ -25,19 +25,32 @@ class BaseHandler(jsonpickle.handlers.BaseHandler):
     def nrestore(self, arg, reset=False):
         return self.context.restore(arg, reset=reset)
 
+    def nflatten(self, arg, reset=False):
+        return self.context.flatten(arg, reset=reset)
+
+
+class NumpyNumber(BaseHandler):
+    def flatten(self, obj, data):
+        data["__reduce__"] = (self.nflatten(type(obj)), [float(obj)])
+        return data
+
+    def restore(self, obj):
+        cls, args = obj['__reduce__']
+        cls = self.nrestore(cls)
+        return cls(args[0])
+
 
 class NumpyArrayHandler(BaseHandler):
     """A jsonpickle handler for numpy (de)serialising arrays."""
-
     def flatten(self, obj, data):
         buf = jsonpickle.util.b64encode(obj.tostring())
         #TODO: should probably also consider including other parameters in future such as byteorder, etc.
         #TODO: see numpy.info(obj) and obj.__reduce__() for details.
-        flatten = self.context.flatten
-        shape = flatten(obj.shape)
+        shape = self.nflatten(obj.shape)
         dtype = str(obj.dtype)
-        args = [shape, dtype, buf]
-        data['__reduce__'] = (flatten(np.ndarray, reset=False), args)
+        order = 'F' if obj.flags.fortran else 'C'
+        args = [shape, dtype, buf, order]
+        data['__reduce__'] = (self.nflatten(np.ndarray), args)
         return data
 
     def restore(self, obj):
@@ -46,18 +59,18 @@ class NumpyArrayHandler(BaseHandler):
         shape = self.nrestore(args[0])
         dtype = np.dtype(self.nrestore(args[1]))
         buf = jsonpickle.util.b64decode(args[2])
-        return cls(shape=shape, dtype=dtype, buffer=buf)
+        order = args[3]
+        return cls(shape=shape, dtype=dtype, buffer=buf, order=order)
 
 
 class PandasTimeSeriesHandler(BaseHandler):
     """A jsonpickle handler for numpy (de)serialising pandas TimeSeries objects."""
 
     def flatten(self, obj, data):
-        flatten = self.context.flatten
-        values = flatten(obj.values)
-        index = flatten(obj.index.values)
+        values = self.nflatten(obj.values)
+        index = self.nflatten(obj.index.values)
         args = [values, index]
-        data['__reduce__'] = (flatten(pd.TimeSeries), args)
+        data['__reduce__'] = (self.nflatten(pd.TimeSeries), args)
         return data
 
     def restore(self, obj):
@@ -73,11 +86,10 @@ class PandasDateTimeIndexHandler(BaseHandler):
     """A jsonpickle handler for numpy (de)serialising pandas DateTimeIndex objects."""
 
     def flatten(self, obj, data):
-        flatten = self.context.flatten
-        values = flatten(obj.values)
-        freq = flatten(obj.freq)
+        values = self.nflatten(obj.values)
+        freq = self.nflatten(obj.freq)
         args = [values, freq]
-        data['__reduce__'] = (flatten(pd.DatetimeIndex), args)
+        data['__reduce__'] = (self.nflatten(pd.DatetimeIndex), args)
         return data
 
     def restore(self, obj):
@@ -96,15 +108,14 @@ def build_index_handler_for_type(index_class):
     class _IndexHandler(BaseHandler):
         """A jsonpickle handler for numpy (de)serialising pandas Index objects."""
         def flatten(self, obj, data):
-            flatten = self.context.flatten
-            values = flatten(obj.values)
+            values = self.nflatten(obj.values)
             args = [values]
-            data['__reduce__'] = (flatten(index_class), args)
+            data['__reduce__'] = (self.nflatten(index_class), args)
             return data
 
         def restore(self, obj):
             cls, args = obj['__reduce__']
-            cls = self.nrestore(cls, reset=False)
+            cls = self.nrestore(cls)
             values = self.nrestore(args[0])
             return cls(data=values)
 
@@ -130,7 +141,7 @@ class PandasDataFrameHandler(BaseHandler):
 
     def restore(self, obj):
         cls, args = obj['__reduce__']
-        cls = self.nrestore(cls, reset=False)
+        cls = self.nrestore(cls)
         values = self.nrestore(args[0])
         index = self.nrestore(args[1])
         columns = self.nrestore(args[2])
@@ -139,6 +150,8 @@ class PandasDataFrameHandler(BaseHandler):
 
 def register_handlers():
     """Call this function to register handlers with jsonpickle module."""
+    NumpyNumber.handles(np.float64)
+    NumpyNumber.handles(np.int64)
     NumpyArrayHandler.handles(np.ndarray)
 
     PandasIndexHandler.handles(pd.Index)
@@ -153,7 +166,17 @@ def register_handlers():
 
 def dumps(obj):
     register_handlers()
-    return jsonpickle.encode(obj).encode("utf-8")
+    return jsonpickle.encode(obj, unpicklable=True).encode("utf-8")
+
+    #from jsonpickle.pickler import _make_backend, Pickler
+    #backend = _make_backend(None)
+    #context = Pickler(unpicklable=True,
+    #                  make_refs=True,
+    #                  keys=False,
+    #                  backend=backend,
+    #                  max_depth=None)
+    #context._mkref = lambda x: True
+    #return backend.encode(context.flatten(obj, reset=False)).encode("utf-8")
 
 
 def loads(obj):
